from rubber.resource import Resource
from rubber.response import Hit, Response
from rubber import settings

HANDLERS = {
    'search': '_search',
    'count': '_count',
    'mapping': '_mapping'
}

class ElasticSearch(object):
    def __init__(self, index_name=None, type=None, auto_index=True, hit_class=Hit, raise_on_error=False):
        self.index_name = index_name
        self.type = type
        self.auto_index = auto_index
        self.hit_class = hit_class
        self.raise_on_error = raise_on_error

    def contribute_to_class(self, model, name):
        if not self.index_name:
            self.index_name = model._meta.app_label

        if not self.type:
            self.type = model._meta.module_name

        self.model = model

        if self.auto_index and not getattr(settings, 'RUBBER_DISABLE_AUTO_INDEX', False):
            try:
                from django.db.models.signals import post_save, post_delete
                post_save.connect(self.django_post_save, sender=model)
                post_delete.connect(self.django_post_delete, sender=model)
            except ImportError, e:
                pass

        setattr(model, name, ElasticSearchDescriptor(self))

    def get(self, pk):
        return Resource(self.makepath(pk), wrapper=Response, raise_on_error=self.raise_on_error).get()

    def put(self, pk, instance):
        return Resource(self.makepath(pk), wrapper=Response, raise_on_error=self.raise_on_error).put(data=instance)

    def delete(self, pk):
        return Resource(self.makepath(pk), wrapper=Response, raise_on_error=self.raise_on_error).delete()

    def django_post_delete(self, sender, instance, **kwargs):
        from rubber.instanceutils import get_pk
        self.delete(get_pk(instance))

    def django_post_save(self, sender, instance, created, **kwargs):
        from rubber.instanceutils import get_pk
        self.put(get_pk(instance), instance)

    def __getattribute__(self, name):
        default_impl = super(ElasticSearch, self).__getattribute__
        try:
            return default_impl(name)
        except AttributeError, e:
            if not name in HANDLERS.keys():
                raise e
            wrapper = Response
            if name == 'search': wrapper=self.wrapsearchresponse
            setattr(self, name, Resource(self.makepath(HANDLERS.get(name)), wrapper=wrapper, raise_on_error=self.raise_on_error))

            return default_impl(name)

    def makepath(self, name):
        tokens = []
        if self.index_name:
            tokens.append(str(self.index_name))
        if self.type:
            tokens.append(str(self.type))
        if name:
            tokens.append(str(name))
        return "/".join(tokens)

    def wrapsearchresponse(self, resp):
        from rubber.response import SearchResponse
        return SearchResponse(resp, hit_class=self.hit_class)

class ElasticSearchDescriptor(object):

    def __init__(self, elasticsearch):
        self.elasticsearch = elasticsearch

    def __get__(self, instance, type=None):
        if instance != None:
            from rubber.resource import InstanceResource
            from rubber.instanceutils import get_pk
            return InstanceResource(instance,
                                    self.elasticsearch.makepath(get_pk(instance)),
                                    wrapper=self.elasticsearch.wrapsearchresponse,
                                    raise_on_error=self.elasticsearch.raise_on_error)
        return self.elasticsearch



from requests.compat import json

def data_to_json(data):

    # convert dicts
    if isinstance(data, dict):
        return json.dumps(data)

    # convert objects that have a to_indexed_json method
    if 'to_indexed_json' in dir(data):
        return data.to_indexed_json()

    # convert django models
    try:
        from django.db import models
        if isinstance(data, models.Model):
            from django.core import serializers
            from django.core.serializers.json import DjangoJSONEncoder
            return json.dumps(serializers.serialize("python", [data])[0]['fields'], cls=DjangoJSONEncoder)
    except ImportError:
        pass

    return data

def get_pk(instance):
    if hasattr(instance, 'pk'):
        return instance.pk
    return None




import requests
from rubber import settings
from rubber.testutils import ResponseMock
from instanceutils import data_to_json

class Resource(object):
    def __init__(self, path, base_url=None, wrapper=None, raise_on_error=False):
        self.path = path
        self.wrapper = wrapper or self._defaultwrapper

        if None == base_url:
            try:
                base_url = settings.RUBBER_ELASTICSEARCH_URL
            except AttributeError:
                pass

        if None == base_url:
            base_url = 'http://localhost:9200/'

        self.base_url = base_url
        self.raise_on_error = raise_on_error

    def _defaultwrapper(self, response):
        return response

    def request(self, method, data=None, **kwargs):
        if settings.RUBBER_MOCK_HTTP_RESPONSE:
            return self.wrapper(ResponseMock(settings.RUBBER_MOCK_HTTP_RESPONSE))
        path = self.base_url + self.path
        try:
            response = requests.request(method, path, data=data_to_json(data), **kwargs)
        except Exception, e:
            if self.raise_on_error:
                raise
            import logging
            logging.exception('Could not perform %s %s' % (method.upper(), path))
            return None
        return self.wrapper(response)

    def get(self, data=None, **kwargs):
        return self.request('GET', data=data, **kwargs)

    def put(self, data=None, **kwargs):
        return self.request('PUT', data=data, **kwargs)

    def post(self, data=None, **kwargs):
        return self.request('POST', data=data, **kwargs)

    def delete(self, data=None, **kwargs):
        return self.request('DELETE', data=data, **kwargs)

    def __call__(self, *args, **kwargs):
        return self.get(*args, **kwargs)

class InstanceResource(Resource):
    def __init__(self, instance, path, base_url=None, **kwargs):
        super(InstanceResource, self).__init__(path, base_url, **kwargs)
        self.instance = instance

    def post(self, **kwargs):
        return super(InstanceResource, self).post(data=self.instance, **kwargs)

    def put(self, **kwargs):
        return super(InstanceResource, self).put(data=self.instance, **kwargs)



import requests

class ResponseMock(requests.models.Response):
    def __init__(self, content='{}'):
        super(ResponseMock, self).__init__()
        from StringIO import StringIO

        self.raw = StringIO(content)
        self.status_code = 200
        self._content = content



class Hit(object):
    def __init__(self, dict_response):
        self.attributes = {}
        for key, val in dict_response.items():
            if isinstance(val, dict):
                self.attributes[key] = Hit(val)
            elif isinstance(val, list):
                self.attributes[key] = [Hit(o) for o in val]
            else:
                self.attributes[key] = val

    def __getattribute__(self, name):
        default_impl = super(Hit, self).__getattribute__
        attributes = default_impl('attributes')
        if attributes.has_key(name):
            return attributes.get(name)
        if not name.startswith('_') and attributes.has_key("_%s"%name):
            return attributes.get("_%s"%name)
        return default_impl(name)
    
import requests
class Response(object):
    """
    A HTTP response object that acts as a proxy on its provided requests.models.Response object
    """
    def __init__(self, response):
        self._response = response

    def __getattribute__(self, attr):
        if attr in ('_response', 'results'):
            return super(Response, self).__getattribute__(attr)
        return getattr(self._response, attr)

class SearchResponse(Response):
    def __init__(self, response, hit_class=Hit):
        super(SearchResponse, self).__init__(response)

        if self._response.json:
            self.results = HitCollection(self._response.json.get('hits'), hit_class=hit_class)
        else:
            self.results = HitCollection({})

class HitCollection(object):
    def __init__(self, dict_response, hit_class=Hit):
        dict_response = dict_response or {}
        self.total = dict_response.get('total')
        self.max_score = dict_response.get('max_score')
        self.hits = []
        for hit_dict in dict_response.get('hits', []):
            self.hits.append(hit_class(hit_dict))

    def __len__(self):
        return len(self.hits)

    def __getitem__(self, i):
        return self.hits[i]



#!/usr/bin/env python
# -*- coding: utf-8 -*-
from setuptools import setup

setup(
    name='rubber',
    version='0.1.7',
    description='Elasticsearch client with Django support.',
    author='Stéphane JAIS',
    author_email='stephane@cantinasoftware.com',
    long_description=open('README.md', 'r').read(),
    url='https://github.com/cantinasoftware/rubber',
    packages=[
        'rubber',
        'tests'
    ],
    requires=[
        'requests(>=0.9.0)',
    ],
    install_requires=[
        'requests>=0.9.0',
    ],
    package_data={
        'rubber': []
    },
    test_suite='tests',
    classifiers=[
        'Development Status :: 3 - Alpha',
        'Environment :: Web Environment',
        'Framework :: Django',
        'Intended Audience :: Developers',
        'License :: OSI Approved :: BSD License',
        'Operating System :: OS Independent',
        'Programming Language :: Python',
        'Topic :: Utilities'
    ],
)
